﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO.Ports;
using System.Net.Sockets;
using TCPSerialProxy.Utils;

namespace TCPSerialProxy
{
    class COMService
    {

        public Boolean InSession
        {
            get
            {
                return sp.IsOpen;
            }
        }
        SerialConfig _serialConfig;
        SerialPort sp;
        public UTF8Encoding utf8Encoding = new UTF8Encoding();
        SendDataCaller dataCallback;
        Socket workerSocket;
        public Boolean dummyComMode;

        public COMService(SerialConfig serialConfig)
        {
            _serialConfig = serialConfig;

            /// Instance the main Serial Port Class
            sp = new SerialPort();

            /// Configure Serial Port
            sp.PortName = _serialConfig.portName;
            sp.BaudRate = _serialConfig.baudRate;
            sp.Parity = _serialConfig.parity;
            sp.DataBits = _serialConfig.dataBits;
            sp.StopBits = _serialConfig.stopBits;

            sp.Handshake = _serialConfig.handshake;
            sp.ReadTimeout = _serialConfig.readTimeout;
            sp.WriteTimeout = _serialConfig.writeTimeout;

            dummyComMode = _serialConfig.dummyComMode;

        }

        public Boolean OpenPort(SendDataCaller sendDataCaller, Socket socket)
        {
            dataCallback = sendDataCaller;
            workerSocket = socket;

            try
            {
                /// Open the SerialPort, thart the thread reading the Raise RTS and DTR
                sp.Open();

                /// Set up Event Handlers
                sp.DataReceived += new SerialDataReceivedEventHandler(DataReceived);

                /// Set Port States
                sp.RtsEnable = true;
                sp.DtrEnable = true;

                return true;
            }
            catch(Exception e)
            {
                Console.WriteLine("Unable to ConnectPort: " + e.Message);

                dataCallback.Invoke(utf8Encoding.GetBytes("ERROR: COM Port Unavailable/Unable to Connect"), workerSocket);

                return false;
            }
        }

        public double dummyValue = 0.001;

        private void DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                SerialPort senderSP = (SerialPort)sender;
                
                if (!dummyComMode)
                {
                    if (senderSP.BytesToRead > 0)
                    {
                        byte[] buffer = new byte[senderSP.BytesToRead];
                        senderSP.Read(buffer, 0, senderSP.BytesToRead);
                        dataCallback.Invoke(buffer, workerSocket);
                    }
                }
                else
                {
                    dataCallback.Invoke(utf8Encoding.GetBytes(" X= 0.900 Y=-0.700 Z= 0.500\n\r"), workerSocket);
                }

               

            }
            catch (Exception ex)
            {
                //Console.WriteLine("DataReceived Error - " + ex.Message);
            }
        }

        public void ClosePort()
        {
            /// Set up Event Handlers
            sp.DataReceived -= new SerialDataReceivedEventHandler(DataReceived);

            /// Close Port
            try
            {
                sp.Close();
            }
            catch (Exception ex)
            {
                Console.WriteLine("DataReceived Error - " + ex.Message);
            }
        }



        public void SendData(byte[] data)
        {
            sp.Write(data, 0, data.Length);
        }

    }
}
