﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.Net;
using TCPSerialProxy.Utils;

namespace TCPSerialProxy
{

    public delegate void SendDataCaller(byte[] data, Socket socket);

    class SerialSocketServer
    {

        private Socket listenerSocket;
        private Socket dumpSocket;
        private COMService comService;
        
        public ASCIIEncoding asciiEncoding = new ASCIIEncoding();

        public SerialSocketServer(SocketConfig socketConf, COMService _comService)
        {
            comService = _comService;

            try
            {
                listenerSocket = new Socket(AddressFamily.InterNetwork,
                                              SocketType.Stream,
                                              ProtocolType.Tcp);

                dumpSocket = new Socket(AddressFamily.InterNetwork,
                                              SocketType.Stream,
                                              ProtocolType.Tcp);

                IPEndPoint ipLocal;

                try
                {
                    ipLocal = new IPEndPoint(IPAddress.Parse(socketConf.ipBinding), socketConf.ipPort);
                    Console.WriteLine("Socket bound to " + socketConf.ipBinding);
                }
                catch
                {
                    ipLocal = new IPEndPoint(IPAddress.Any, socketConf.ipPort);
                    Console.WriteLine("Socket bound to any IP.");
                }

                listenerSocket.Bind(ipLocal);
                listenerSocket.Listen(4);
                listenerSocket.BeginAccept(new AsyncCallback(OnClientConnect), null);

                Console.WriteLine("Listening on port " + socketConf.ipPort);
            }
            catch (SocketException se)
            {
                Console.WriteLine(se.Message);
            }
        }

        public void OnClientConnect(IAsyncResult asyn)
        {
            if (comService.InSession == false)
            {
                /// If the worker Socket is not conected pass over the connection
                StartDataSession(listenerSocket.EndAccept(asyn));
            }
            else
            {
                /// If the worker socket is busy, reject the connection
                /// A rejection notice is sent to the client then the socket is closed
                RejectClient(listenerSocket.EndAccept(asyn));
            }

            listenerSocket.BeginAccept(new AsyncCallback(OnClientConnect), null);
        }

        private void RejectClient(Socket socketToReject)
        {
            System.Diagnostics.Debugger.Log(0, "1", "Client Rejected: Only 1 Client can connect at once - " + socketToReject.RemoteEndPoint.ToString() + "\n");
            socketToReject.Send(asciiEncoding.GetBytes("ERROR: Sorry, this server only allows one client to be connected at once. Exiting..."));
            socketToReject.Disconnect(true);
        }

        protected void StartDataSession(Socket workerSocket)
        {
            Console.WriteLine("Client Connected - Starting DataSession with " + workerSocket.RemoteEndPoint.ToString());

            comService.OpenPort(new SendDataCaller(this.SendDataMethod), workerSocket);
            SocketBeginReceive(workerSocket);
        }

        private void SocketBeginReceive(Socket workerSocket)
        {
            SocketPacket socketPacket = new SocketPacket();
            socketPacket.currentWorker = workerSocket;

            try
            {
                workerSocket.BeginReceive(
                    socketPacket.dataBuffer,
                    0,
                    socketPacket.dataBuffer.Length,
                    SocketFlags.None,
                    new AsyncCallback(DataReceived),
                    socketPacket
                    );
            }
            catch (SocketException e)
            {
                System.Diagnostics.Debugger.Log(0, "1", "Socket Exception: " + e.Message);

                EndDataSession(workerSocket);

            }
        }

        private void DataReceived(IAsyncResult asyn)
        {
            SocketPacket socketPacket = (SocketPacket)asyn.AsyncState;

            try
            {
                /// Try and End Reception - Also Used for detecting a closed connection
                socketPacket.currentWorker.EndAccept(asyn);

                /// Send the data to COM
                comService.SendData(socketPacket.dataBuffer);

                /// Listen for More Data
                SocketBeginReceive(socketPacket.currentWorker);
            }
            catch (ArgumentException)
            {
                EndDataSession(socketPacket.currentWorker);
            }
        }

        public void SendDataMethod(byte[] data, Socket socket)
        {
            socket.Send(data);
        }

        private void EndDataSession(Socket workerSocket)
        {
            /// Close the various sockets and ports
            comService.ClosePort();

            workerSocket.Disconnect(true);  /// I thinkmore work is required here

            Console.WriteLine("Session Ended");

        }

    }
}
